function [perf] = evaluate_cluster(fname, models, param)
load(fname);

num_months = length(models);
num_relation = length(R{1}.relation);

%num_months = 2;

total_error = zeros(num_months, 1);
total_maxtrace = zeros(num_months, 1);
error = zeros(num_relation, num_months);
maxtrace = zeros(num_relation, num_months);

if ~isfield(param, 'normalize')
    param.normalize = false;
end

if ~isfield(param, 'weights')
    param.weights = ones(num_relation, 1);
end
    
w = param.weights;
for month = 1:num_months
    if param.normalize == true
        num_paper = size(R{month}.relation{1}, 1);
        for I = 1:length(w)
            weights(I) = w(I) /(num_paper* num_entity{month}(I));
        end
    else
        weights = w;
    end

    
    [total_error(month), error(:, month), total_maxtrace(month), maxtrace(:, month)]=evaluate_cluster_one(R{month}, activeIndex{month}, models{month}, weights);
    perf.total_error = total_error;
    perf.error = error;
    perf.total_maxtrace = total_maxtrace;
    perf.maxtrace = maxtrace;
end



function [total_error, error, total_maxtrace, maxtrace] =  evaluate_cluster_one(R, activeIndex, model, weight)
%[R, activeIndex] = shrink_relation(R);

num = length(R.relation);
error = zeros(num,1);
maxtrace = zeros(num,1);
for I = 1:num
     idx = model.idx{I}(activeIndex{I});
     indicator = construct_indicator(idx);
     [error(I), maxtrace(I)] = calculate_error(R.relation{I}, indicator);
end

w(1, 1:num) = weight(1:num);
total_error = w*error;
total_maxtrace = w*maxtrace;


function [error, temp2] = calculate_error(relation, indicator)
temp1 = norm(relation, 'fro')^2;
temp2 = norm(relation*indicator, 'fro')^2;
error = temp1 - temp2;


function [indicator] = construct_indicator(idx)
% given an cluster membership assignment, construct a corresponding
% cluster indicator matrix
numC = max(idx);
n = length(idx);
indicator = sparse(n, numC);


for I = 1:1:numC(1,1)
    %disp(I)
    index = find(idx==I);
    if ~isempty(index)
       num = length(index);
       indicator(index, I) = sqrt(1/num) * ones(num,1);
    end
end

function [total_error, error] = random_evaluate(R, numC)
[R, activeIndex] = shrink_relation(R);

num = length(R.relation);
error = zeros(num, 1);
for I = 1:num
    idx = ceil((numC(I)+1)*rand(length(activeIndex{I}), 1))-1;
    indicator = construct_indicator(idx);
    error(I) = calculate_error(R.relation{I}, indicator);
end

total_error = sum(error);


