function [model] = offline_one(R, model, temporal,param,perform_kmeans)
% perform cluster indicator update at one timestamp
% R: the snapshot of multi-mode network at this timestamp
% model: the clustering results of different modes at current timestamp (computed in the previous iteration)
% temporal.param: related parameters concerning temporal information
%                 if 'prior' or 'after' is presented, then the clustering results of neighboring timestamps
%	          will be included for computation
% perform_keans: whether or not to perform final kmeans after we update community indicator
% 
% Return model: which contains updated clustering indicator of each mode at one timestamp

numC = param.numC;
weight_temporal = param.weight_temporal;
weights = param.weights;

% this should be done before clustering
num_relation = length(R.relation);
activeIndex = model.activeIndex;
num_original  = model.num_original;

paperIndex = num_relation+1;


% compute the clustering result for authors, words, and venues
cluster = model.cluster;
for rIndex = 1:num_relation
    P = R.relation{rIndex}' * model.cluster{paperIndex} *  sqrt(weights(rIndex));
    if isfield(temporal, 'prior') && isfield(temporal, 'after')
        P = [P, weight_temporal(rIndex)*temporal.prior{rIndex}, ...
            weight_temporal(rIndex)*temporal.after{rIndex}];
    elseif isfield(temporal, 'prior')
        P = [P, weight_temporal(rIndex)*temporal.prior{rIndex}];
    elseif isfield(temporal, 'after')
        P = [P, weight_temporal(rIndex)*temporal.after{rIndex}];
    end
    
    [cluster{rIndex}, tempD] = svds(P, numC(rIndex));
end

% compute the clustering result for papers (no temporal information to consider)
clear P;
P = [];
for rIndex = 1:num_relation
    P = [P, sqrt(weights(rIndex)) * (R.relation{rIndex} * model.cluster{rIndex})];
end
[cluster{paperIndex}, tempD] = svds(P, numC(paperIndex));

% ------ calculate the objective --------

obj = 0;
temp = cluster{paperIndex}';
for rIndex = 1:num_relation
    obj = obj + weights(rIndex) * norm( temp* R.relation{rIndex} * cluster{rIndex}, 'fro')^2;
    if  isfield(temporal, 'prior')
        obj = obj + weight_temporal(rIndex) * norm( temporal.prior{rIndex}' * cluster{rIndex}, 'fro')^2;
    end
end


model.obj = obj;

if perform_kmeans==true
    
    % --- calculate the core represnetation ---
    for rIndex = 1:num_relation
        model.core{rIndex} = cluster{paperIndex}' * R.relation{rIndex} * cluster{rIndex};
    end
    
    for rIndex = 1:length(cluster)
        temp = kmeans(cluster{rIndex}, numC(rIndex), 'EmptyAction', 'singleton');
        % restore to the original space

        if rIndex ~= paperIndex
            model.idx{rIndex} = sparse(num_original(rIndex), 1);
            model.idx{rIndex}(activeIndex{rIndex}) = temp;
        else
            model.idx{rIndex}  = temp;
        end
    end
    model.param = param;
    
    
end

model.cluster = cluster;
if ~isfield(model, 'iter')
    model.iter = 1;
else
    model.iter = model.iter+1;
end
