function [model] = online_one(R, model, param, temporal)
% a function to implement the online relational
if nargin < 4
    temporal.NULL = true;
end

if isfield (param, 'weight_temporal')
    weight_temporal = param.weight_temporal;
end

activeIndex = model.activeIndex;
num_original  = model.num_original;
num_relation = length(R.relation);
paperIndex = num_relation +1;

weights = param.weights;
numC = param.numC;
tol = param.tol;
maxIter = param.maxIter;


iter = 0;
pre_obj= 0;
diff = inf;
cluster = model.cluster;
obj_history = [];
while iter< maxIter && diff > tol


    for rIndex = 1:num_relation
        P = sqrt(weights(rIndex)) * (R.relation{rIndex}' * cluster{paperIndex});
        if isfield(temporal, 'prior')
            P = [P, weight_temporal(rIndex) * temporal.prior{rIndex}];
        end
        [cluster{rIndex}, tempD] = svds(P, numC(rIndex));
    end

    clear P;
    P = [];
    for rIndex = 1:num_relation
        P = [P, sqrt(weights(rIndex)) * (R.relation{rIndex} * model.cluster{rIndex})];
    end
    [cluster{paperIndex}, tempD] = svds(P, numC(paperIndex));

    obj = 0;
    temp = cluster{paperIndex}';
    for rIndex = 1:num_relation
        obj = obj + weights(rIndex) * norm( temp* R.relation{rIndex} * cluster{rIndex}, 'fro')^2;
        if  isfield(temporal, 'prior')
            temp = weight_temporal(rIndex) * norm( temporal.prior{rIndex}' * cluster{rIndex}, 'fro')^2;
            obj = obj + temp;
        end
    end
    diff = (obj - pre_obj)/obj;
    pre_obj = obj
    obj_history = [obj_history, obj];
    iter = iter+1;
end

if ~isfield(param, 'kmeans') || param.kmeans == true
    % --- calculate the core represnetation ---
    for rIndex = 1:num_relation
        model.core{rIndex} = cluster{paperIndex}' * R.relation{rIndex} * cluster{rIndex};
    end

    %% perform kmeans
    disp('perform kmeans .......');
    for rIndex = 1:length(cluster)
        temp = kmeans(cluster{rIndex}, numC(rIndex), 'EmptyAction', 'singleton');
        % restore to the original space
        if rIndex ~= paperIndex
            model.idx{rIndex} = sparse(num_original(rIndex), 1);
            model.idx{rIndex}(activeIndex{rIndex}) = temp;
        else
            model.idx{rIndex}  = temp;
        end
    end
end

model.cluster = cluster;
model.param = param;
model.iter = iter;
model.obj = obj;
model.obj_history = obj_history;
